@@ -2,7 +2,7 @@ package Net::DNS::RR::SVCB;
 
 use strict;
 use warnings;
-our $VERSION = (qw$Id: SVCB.pm 1965 2024-02-14 09:19:32Z willem $)[2];
+our $VERSION = (qw$Id: SVCB.pm 2003 2025-01-21 12:06:06Z willem $)[2];
 
 use base qw(Net::DNS::RR);
 
@@ -19,18 +19,25 @@ use Net::DNS::DomainName;
 use Net::DNS::RR::A;
 use Net::DNS::RR::AAAA;
 use Net::DNS::Text;
+use MIME::Base64;
 
 
 my %keybyname = (
-	mandatory	  => 'key0',
-	alpn		  => 'key1',
+	mandatory	       => 'key0',			# RFC9460(8)
+	alpn		       => 'key1',			# RFC9460(7.1)
+	'no-default-alpn'      => 'key2',			# RFC9460(7.1)
+	port		       => 'key3',			# RFC9460(7.2)
+	ipv4hint	       => 'key4',			# RFC9460(7.3)
+	ech		       => 'key5',			# RFC9460
+	ipv6hint	       => 'key6',			# RFC9460(7.3)
+	dohpath		       => 'key7',			# RFC9461
+	ohttp		       => 'key8',			# RFC9540(4)
+	'tls-supported-groups' => 'key9',
+	);
+
+my %boolean = (
 	'no-default-alpn' => 'key2',
-	port		  => 'key3',
-	ipv4hint	  => 'key4',
-	ech		  => 'key5',
-	ipv6hint	  => 'key6',
-	dohpath		  => 'key7',				# draft-schwartz-svcb-dns
-	ohttp		  => 'key8',				# draft-pauly-ohai-svcb-config
+	ohttp		  => 'key8',
 	);
 
 
@@ -38,7 +45,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 	my ( $self, $data, $offset ) = @_;
 
 	my $limit = $self->{rdlength};
-	my $rdata = substr $$data, $offset, $limit;
+	my $rdata = $self->{rdata} = substr $$data, $offset, $limit;
 	$self->{SvcPriority} = unpack 'n', $rdata;
 	( $self->{TargetName}, $offset ) = Net::DNS::DomainName->decode( \$rdata, 2 );
 
@@ -50,10 +57,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 		push @$params, ( $key, substr $rdata, $start, $size );
 		$offset = $next;
 	}
-	unless ( $offset == $limit ) {
-		$self->{unparsed} = substr $rdata, $offset;
-		die $self->type . ': corrupt RDATA';
-	}
+	die $self->type . ': corrupt RDATA' unless $offset == $limit;
 	return;
 }
 
@@ -61,6 +65,7 @@ sub _decode_rdata {			## decode rdata from wire-format octet string
 sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
+	return $self->{rdata} if $self->{rdata};
 	my @packed = pack 'n a*', $self->{SvcPriority}, $self->{TargetName}->encode;
 	my $params = $self->{SvcParams} || [];
 	my @params = @$params;
@@ -76,33 +81,33 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $priority = $self->{SvcPriority};
-	my $target   = $self->{TargetName}->string;
-	my $params   = $self->{SvcParams} || [];
-	return ( $priority, $target ) unless $priority;
+	my @rdata = unpack 'H4', pack 'n', $self->{SvcPriority};
 
 	my $encode = $self->{TargetName}->encode();
 	my $length = 2 + length $encode;
 	my @target = grep {length} split /(\S{32})/, unpack 'H*', $encode;
-	my @rdata  = unpack 'H4', pack 'n', $priority;
-	push @rdata, "\t; $priority\n";
-	push @rdata, shift @target;
-	push @rdata, join '', "\t; ", substr( $target, 0, 40 ), "\n";
+	my $target = substr $self->{TargetName}->string, 0, 40;
+	push @rdata, join '', shift(@target), "\t; $target\n";
 	push @rdata, @target;
 
+	my $params = $self->{SvcParams} || [];
 	my @params = @$params;
 	while (@params) {
 		my $key = shift @params;
 		my $val = shift @params;
-		push @rdata, "\n";
-		push @rdata, "; key$key=...\n" if $key > 25;
-		push @rdata, unpack 'H4H4',    pack( 'n2', $key, length $val );
-		push @rdata, split /(\S{32})/, unpack 'H*', $val;
+		push @rdata, "\n", unpack 'H4H4', pack( 'n2', $key, length $val );
+		my @hex = grep {length} split /(\S{32})/, unpack 'H*', $val;
+		push @rdata, shift @hex if @hex;
+		push @rdata, "\t; key$key\n" unless $key < 16;
+		push @rdata, @hex;
 		$length += 4 + length $val;
 	}
-	if ( $self->{unparsed} ) {
-		my @hex = split /(\S{32})/, unpack 'H*', $self->{unparsed};
-		push @rdata, "\n", shift(@hex), "\t; corrupt RDATA\n", @hex;
+	if ( $self->{rdata} ) {
+		if ( my $corrupt = substr $self->{rdata}, $length ) {
+			my ( $hex, @hex ) = grep {length} split /(\S{32})/, unpack 'H*', $corrupt;
+			push @rdata, "\n$hex\t; corrupt RDATA\n", @hex;
+			$length += length $corrupt;
+		}
 	}
 	return ( "\\# $length", @rdata );
 }
@@ -119,18 +124,27 @@ sub _parse_rdata {			## populate RR from rdata in argument list
 		for ($svcparam) {
 			my @value;
 			if (/^key\d+=(.*)$/i) {
-				push @value, length($1) ? $1 : shift @argument;
-			} elsif (/^[^=]+=(.*)$/) {
 				local $_ = length($1) ? $1 : shift @argument;
 				s/^"([^"]*)"$/$1/;		# strip enclosing quotes
+				push @value, $_;
+			} elsif (/^[^=]+=(.*)$/) {
+				local $_ = length($1) ? $1 : shift @argument;
+				die <<"Amen" if /\\092[,\\]/;
+SVCB:	Please use standard RFC1035 escapes
+	RFC9460 double-escape nonsense not implemented
+Amen
+				s/^"([^"]*)"$/$1/;		# strip enclosing quotes
+				s/\\,/\\044/g;			# disguise (RFC1035) escaped comma
 				push @value, split /,/;
 			} else {
-				push @value, '' unless $keybyname{lc $_};    # empty | Boolean
+				push @value, '' unless $keybyname{$_};	  # unregistered boolean key
 			}
 
-			s/[-]/_/g;				# extract identifier
-			m/^([^=]+)/;
-			$self->$1(@value);
+			m/^([^=]+)/;				# extract identifier
+			my $key = $1;
+			push @value, 1 if $boolean{$key};
+			$key =~ s/[-]/_/g;
+			$self->$key(@value);
 		}
 	}
 	return;
@@ -198,7 +212,7 @@ sub alpn {				## alpn=h3,h2,...
 
 sub no_default_alpn {			## no-default-alpn	(Boolean)
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key2( ( defined(wantarray) ? () : '' ), @value );
+	return $self->key2( _boolean(@value) );
 }
 
 sub port {				## port=1234
@@ -211,9 +225,9 @@ sub ipv4hint {				## ipv4hint=192.0.2.1,...
 	return $self->key4( _ipv4(@value) );
 }
 
-sub ech {				## Format not specified
+sub ech {				## ech=base64
 	my ( $self, @value ) = @_;
-	return $self->key5(@value);				# RESERVED
+	return $self->key5( map { _base64($_) } @value );
 }
 
 sub ipv6hint {				## ipv6hint=2001:DB8::1,...
@@ -226,21 +240,44 @@ sub dohpath {				## dohpath=/dns-query{?dns}
 	return $self->key7(@value);
 }
 
-sub ohttp {				## ohttp	(Boolean)
+sub ohttp {				## ohttp
 	my ( $self, @value ) = @_;				# uncoverable pod
-	return $self->key8( ( defined(wantarray) ? () : '' ), @value );
+	return $self->key8( _boolean(@value) );
+}
+
+sub tls_supported_groups {		## tls_supported_groups=29,23
+	my ( $self, @value ) = @_;				# uncoverable pod
+	return $self->key9( _integer16(@value) );
 }
 
 
 ########################################
 
 
-sub _presentation {			## render octet string(s) in presentation format
+sub _presentation {			## represent octet string(s) using local charset
 	my @arg = @_;
-	my $raw = scalar(@arg) ? join( '', @arg ) : return ();
+	my $raw = scalar(@arg) ? join( '', @arg ) : return ();	# concatenate arguments
 	return Net::DNS::Text->decode( \$raw, 0, length($raw) )->string;
 }
 
+sub _boolean {
+	my @arg = @_;
+	return @arg unless scalar @arg;				# read key
+	my $arg = shift @arg;
+	return $arg unless defined $arg;			# delete key.
+	return ( $arg ? '' : undef, @arg );			# set key
+}
+
+sub _string {
+	my @arg = @_;
+	return _presentation( map { Net::DNS::Text->new($_)->encode() } @arg );
+}
+
+sub _base64 {
+	my @arg = @_;
+	return _presentation( map { MIME::Base64::decode($_) } @arg );
+}
+
 sub _integer16 {
 	my @arg = @_;
 	return _presentation( map { pack( 'n', $_ ) } @arg );
@@ -256,17 +293,6 @@ sub _ipv6 {
 	return _presentation( map { Net::DNS::RR::AAAA::address( {}, $_ ) } @arg );
 }
 
-sub _string {
-	my @arg = @_;
-	local $_ = join ',', @arg;				# reassemble argument string
-	s/\\,/\\044/g;						# disguise (RFC1035) escaped comma
-	die <<"QQ" if /\\092,|\\092\\092/;
-SVCB:	Please use standard RFC1035 escapes
-	RFC9460 double-escape nonsense not implemented
-QQ
-	return _presentation( map { Net::DNS::Text->new($_)->encode() } split /,/ );
-}
-
 
 sub AUTOLOAD {				## Dynamic constructor/accessor methods
 	my ( $self, @argument ) = @_;
@@ -286,15 +312,15 @@ sub AUTOLOAD {				## Dynamic constructor/accessor methods
 		delete $svcparams{$key} unless defined $arg;
 		die( $self->type . qq[: duplicate SvcParam "key$key"] ) if defined $svcparams{$key};
 		die( $self->type . qq[: invalid SvcParam "key$key"] )	if $key > 65534;
-		$svcparams{$key}   = Net::DNS::Text->new("$arg")->raw if defined $arg;
+		die( $self->type . qq[: unexpected "key$key" value] )	if scalar @argument;
+		delete $self->{rdata};
+		$svcparams{$key} = Net::DNS::Text->new("$arg")->raw if defined $arg;
 		$self->{SvcParams} = [map { ( $_, $svcparams{$_} ) } sort { $a <=> $b } keys %svcparams];
-		die( $self->type . qq[: unexpected number of values for "key$key"] ) if scalar @argument;
 	} else {
 		die( $self->type . qq[: no value specified for "key$key"] ) unless defined wantarray;
 	}
 
-	my $value = $svcparams{$key};
-	return defined($value) ? _presentation($value) : $value;
+	return $svcparams{$key};
 }
 
 ########################################
@@ -306,8 +332,8 @@ __END__
 
 =head1 SYNOPSIS
 
-    use Net::DNS;
-    $rr = Net::DNS::RR->new('name HTTPS SvcPriority TargetName SvcParams');
+	use Net::DNS;
+	$rr = Net::DNS::RR->new('name SVCB  SvcPriority TargetName SvcParams');
 
 =head1 DESCRIPTION
 
@@ -328,8 +354,8 @@ other unpredictable behaviour.
 
 =head2 SvcPriority
 
-    $svcpriority = $rr->svcpriority;
-    $rr->svcpriority( $svcpriority );
+	$svcpriority = $rr->svcpriority;
+	$rr->svcpriority( $svcpriority );
 
 The priority of this record
 (relative to others, with lower values preferred). 
@@ -337,8 +363,8 @@ A value of 0 indicates AliasMode.
 
 =head2 TargetName
 
-    $rr->targetname( $targetname );
-    $effecivetarget = $rr->targetname;
+	$rr->targetname( $targetname );
+	$effecivetarget = $rr->targetname;
 
 The domain name of either the alias target (for AliasMode)
 or the alternative endpoint (for ServiceMode).
@@ -351,33 +377,34 @@ owner name of this record must be used as the effective TargetName.
 
 =head2 mandatory, alpn, no-default-alpn, port, ipv4hint, ech, ipv6hint
 
-    $rr = Net::DNS::RR->new( 'svcb.example. SVCB 1 svcb.example. port=1234' );
+	$rr = Net::DNS::RR->new( 'svcb.example. SVCB 1 svcb.example. port=1234' );
 
-    $rr->port(1234);
-    $string = $rr->port();	# \004\210
-    $rr->key3($string);
+	$rr->port(1234);
+	$octets = $rr->port();	# 0x04 0xD2
+	$octets = $rr->key3();
 
 Constructor methods for mnemonic SvcParams prescribed by RFC9460.
-When invoked without arguments, the methods return the presentation format
-value for the underlying key.
+When invoked without arguments, the methods return the value
+of the underlying key as an uninterpreted octet string.
 The behaviour with undefined arguments is not specified.
 
 =head2 keyNN
 
-    $keynn = $rr->keyNN;
-    $rr->keyNN( $keynn );
-    $rr->keyNN( undef );
+	$keynn = $rr->keyNN;
+	$rr->keyNN( $keynn );
+	$rr->keyNN( undef );
 
 Generic constructor and accessor methods for SvcParams.
 The key index NN is a decimal integer in the range 0 .. 65535.
-The method argument and returned value are both presentation format strings.
+The method argument is a presentation format character string.
+The returned value is an uninterpreted octet string.
 The method returns the undefined value if the key is not present.
-The specified key will be deleted if the value is undefined.
+The specified key will be deleted if the argument is undefined.
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2020-2022 Dick Franks. 
+Copyright (c)2020-2024 Dick Franks. 
 
 All rights reserved.
 
@@ -406,8 +433,8 @@ DEALINGS IN THE SOFTWARE.
 =head1 SEE ALSO
 
 L<perl> L<Net::DNS> L<Net::DNS::RR>
-L<RFC9460|https://tools.ietf.org/html/rfc9460>
+L<RFC9460(2)|https://iana.org/go/rfc9460#section-2>
 
-L<Service Parameter Keys|https://www.iana.org/assignments/dns-svcb>
+L<Service Parameter Keys|https://iana.org/assignments/dns-svcb>
 
 =cut
